@@ -1,6 +1,6 @@
 /****************************************************************************
  *
- *  $Id: linphonec.c,v 1.51 2006/08/21 12:49:59 smorlat Exp $
+ *  $Id: linphonec.c,v 1.52 2007/01/10 14:11:24 smorlat Exp $
  *
  *  Copyright (C) 2006  Sandro Santilli <strk@keybit.net>
  *  Copyright (C) 2002  Florian Winterstein <flox@gmx.net>
@@ -107,6 +107,7 @@ static char last_in_history[256];
 //auto answer (-a) option
 static bool_t auto_answer=FALSE;
 static bool_t answer_call=FALSE;
+static bool_t video_enabled=FALSE;
 LPC_AUTH_STACK auth_stack;
 static int trace_level = 0;
 static char *logfile_name = NULL;
@@ -354,7 +355,8 @@ linphonec_init(int argc, char **argv)
 	 */
 	linphone_core_init (&linphonec, &linphonec_vtable, configfile_name,
 			    NULL);
-
+	linphone_core_enable_video(&linphonec,video_enabled);
+	if (!video_enabled) printf("Warning: video is disabled in linphonec. Run with -V option to enable it.\n");
 	/*
 	 * Initialize readline
 	 */
@@ -484,14 +486,15 @@ void
 print_usage (int exit_status)
 {
 	fprintf (stdout, "\n\
-usage: linphonec [-c file] [-s sipaddr] [-a] [-d level -l logfile]\n\
+usage: linphonec [-c file] [-s sipaddr] [-a] [-V] [-d level ] [-l logfile]\n\
        linphonec -v\n\
 \n\
   -c  file             specify path of configuration file.\n\
   -d  level            be verbose. 0 is no output. 6 is all output\n\
   -l  logfile          specify the log file for your SIP phone\n\
   -s  sipaddress       specify the sip call to do at startup\n\
-  -a		       enable auto answering for incoming calls\n\
+  -a                   enable auto answering for incoming calls\n\
+  -V                   enable video (disabled by default)\n\
   -v or --version      display version and exits.\n");
 
   	exit(exit_status);
@@ -713,6 +716,10 @@ linphonec_parse_cmdline(int argc, char **argv)
                 {
                         auto_answer = TRUE;
                 }
+		else if (strncmp ("-V", argv[arg_num], 2) == 0)
+                {
+                        video_enabled = TRUE;
+                }
 		else if ((strncmp ("-v", argv[arg_num], 2) == 0)
 			 ||
 			 (strncmp
@@ -921,6 +928,9 @@ lpc_strip_blanks(char *input)
 /****************************************************************************
  *
  * $Log: linphonec.c,v $
+ * Revision 1.52  2007/01/10 14:11:24  smorlat
+ * add --video to linphonec.
+ *
  * Revision 1.51  2006/08/21 12:49:59  smorlat
  * merged several little patches.
  *
