@@ -23,7 +23,7 @@
  */
  
  #include "linphonecore.h"
- #include "misc.h"
+ #include "private.h"
  #include <eXosip.h>
  
  LinphoneChatRoom * linphone_core_create_chat_room(LinphoneCore *lc, const char *to)
@@ -31,11 +31,11 @@
 	 char *real_url=NULL;
 	 osip_from_t *parsed_url=NULL;
 	 if (linphone_core_interpret_url(lc,to,&real_url,&parsed_url)){
-	 	LinphoneChatRoom *cr=g_new0(LinphoneChatRoom,1);
+	 	LinphoneChatRoom *cr=ms_new0(LinphoneChatRoom,1);
 	 	cr->lc=lc;
 	 	cr->peer=real_url;
 		cr->peer_url=parsed_url;
-	 	lc->chatrooms=g_list_append(lc->chatrooms,(gpointer)cr);
+	 	lc->chatrooms=ms_list_append(lc->chatrooms,(void *)cr);
 	 	return cr;
 	 }
 	 return NULL;
@@ -44,9 +44,9 @@
  
  void linphone_chat_room_destroy(LinphoneChatRoom *cr){
 	 LinphoneCore *lc=cr->lc;
-	 lc->chatrooms=g_list_remove(lc->chatrooms,(gpointer) cr);
+	 lc->chatrooms=ms_list_remove(lc->chatrooms,(void *) cr);
 	 osip_from_free(cr->peer_url);
-	 g_free(cr->peer);
+	 ms_free(cr->peer);
  }
  
 void linphone_chat_room_send_message(LinphoneChatRoom *cr, const char *msg){
@@ -54,7 +54,7 @@ void linphone_chat_room_send_message(LinphoneChatRoom *cr, const char *msg){
 	eXosip_message(cr->peer,(char*)identity, (char*)linphone_core_get_route(cr->lc),(char*)msg);
 }
 
-gboolean linphone_chat_room_matches(LinphoneChatRoom *cr, osip_from_t *from){
+bool_t linphone_chat_room_matches(LinphoneChatRoom *cr, osip_from_t *from){
 	if (strcmp(cr->peer_url->url->username,from->url->username)==0) return TRUE;
 	return FALSE;
 }
@@ -64,18 +64,18 @@ void linphone_chat_room_text_received(LinphoneChatRoom *cr, LinphoneCore *lc, co
 }
 
 void linphone_core_text_received(LinphoneCore *lc, const char *from, const char *msg){
-	GList *elem;
+	MSList *elem;
 	LinphoneChatRoom *cr=NULL;
 	char *cleanfrom;
 	osip_from_t *from_url;
 	osip_from_init(&from_url);
 	if (osip_from_parse(from_url,from)<0){
-		g_warning("linphone_core_text_received: could not parse %s.",from);
+		ms_warning("linphone_core_text_received: could not parse %s.",from);
 		osip_from_free(from_url);
 		return;
 	}
 	from_2char_without_params(from_url,&cleanfrom);
-	for(elem=lc->chatrooms;elem!=NULL;elem=g_list_next(elem)){
+	for(elem=lc->chatrooms;elem!=NULL;elem=ms_list_next(elem)){
 		cr=(LinphoneChatRoom*)elem->data;
 		if (linphone_chat_room_matches(cr,from_url)){
 			break;
@@ -92,9 +92,9 @@ void linphone_core_text_received(LinphoneCore *lc, const char *from, const char
 }
 
 
-void linphone_chat_room_set_user_data(LinphoneChatRoom *cr, gpointer ud){
+void linphone_chat_room_set_user_data(LinphoneChatRoom *cr, void * ud){
 	cr->user_data=ud;
 }
-gpointer linphone_chat_room_get_user_data(LinphoneChatRoom *cr){
+void * linphone_chat_room_get_user_data(LinphoneChatRoom *cr){
 	return cr->user_data;
 }
