@@ -492,7 +492,7 @@ SupportLevel linphone_payload_is_supported(LinphoneCore *lc, sdp_payload_t *payl
 					ubw);
 			}
 			/* but anyway give our download bandwidth constraint*/
-			payload->b_as_bandwidth=dbw;
+			payload->b_as_bandwidth=(dbw>0) ? dbw : 0;
 			if (payload->a_fmtp!=NULL){
 				payload_type_set_send_fmtp(rtppayload,payload->a_fmtp);
 			}
