@@ -35,12 +35,16 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #endif
 #endif
 
+
+static const char *liblinphone_version=LIBLINPHONE_VERSION;
+
 #include "enum.h"
 
 void linphone_core_enable_video_preview(LinphoneCore *lc, bool_t val);
 bool_t linphone_core_video_preview_enabled(const LinphoneCore *lc);
 void linphone_core_get_local_ip(LinphoneCore *lc, const char *dest, char *result);
 static void apply_nat_settings(LinphoneCore *lc);
+static void toggle_video_preview(LinphoneCore *lc, bool_t val);
 
 /* relative path where is stored local ring*/
 #define LOCAL_RING "rings/oldphone.wav"
@@ -119,6 +123,11 @@ void linphone_call_destroy(LinphoneCall *obj)
 	ms_free(obj);
 }
 
+/*prevent a gcc bug with %c*/
+static size_t my_strftime(char *s, size_t max, const char  *fmt,  const struct tm *tm){
+	return strftime(s, max, fmt, tm);
+}
+
 LinphoneCallLog * linphone_call_log_new(LinphoneCall *call, char *from, char *to){
 	LinphoneCallLog *cl=ms_new0(LinphoneCallLog,1);
 	struct tm loctime;
@@ -129,7 +138,7 @@ LinphoneCallLog * linphone_call_log_new(LinphoneCall *call, char *from, char *to
 #else
 	localtime_r(&call->start_time,&loctime);
 #endif
-	strftime(cl->start_date,sizeof(cl->start_date),"%c",&loctime);
+	my_strftime(cl->start_date,sizeof(cl->start_date),"%c",&loctime);
 	cl->from=from;
 	cl->to=to;
 	return cl;
@@ -442,7 +451,6 @@ void video_config_read(LinphoneCore *lc)
 	tmp=lp_config_get_int(lc->config,"video","enabled",1);
 #ifdef VIDEO_ENABLED
 	linphone_core_enable_video(lc,tmp);
-	linphone_core_enable_video_preview(lc,tmp);
 #endif
 }
 
@@ -477,7 +485,7 @@ void linphone_core_set_download_bandwidth(LinphoneCore *lc, int bw){
 		lc->dw_audio_bw=80;
 	}else if (bw>=128){
 		if (linphone_core_video_enabled(lc))
-			lc->dw_audio_bw=40;
+			lc->dw_audio_bw=30;
 		else
 			lc->dw_audio_bw=bw;
 	}else{
@@ -496,7 +504,7 @@ void linphone_core_set_upload_bandwidth(LinphoneCore *lc, int bw){
 		lc->up_audio_bw=80;
 	}else if (bw>=128){
 		if (linphone_core_video_enabled(lc))
-			lc->up_audio_bw=40;
+			lc->up_audio_bw=30;
 		else 
 			lc->up_audio_bw=bw;
 	}else{
@@ -513,6 +521,10 @@ int linphone_core_get_upload_bandwidth(const LinphoneCore *lc){
 	return lc->net_conf.upload_bw;
 }
 
+const char * linphone_core_get_version(void){
+	return liblinphone_version;
+}
+
 void
 linphone_core_init (LinphoneCore * lc, const LinphoneCoreVTable *vtable, const char *config_path, void * userdata)
 {
@@ -543,13 +555,11 @@ linphone_core_init (LinphoneCore * lc, const LinphoneCoreVTable *vtable, const c
 	lc->rsvp_enable = 1;
 	lc->rpc_enable = 0;
 #endif
-
 	sound_config_read(lc);
 	net_config_read(lc);
-	sip_config_read(lc); /* this will start eXosip*/
-	
 	rtp_config_read(lc);
 	codecs_config_read(lc);
+	sip_config_read(lc); /* this will start eXosip*/
 	video_config_read(lc);
 	//autoreplier_config_init(&lc->autoreplier_conf);
 	
@@ -876,7 +886,13 @@ void linphone_core_iterate(LinphoneCore *lc)
 			}
 		}
 	}
-
+	if (linphone_core_video_preview_enabled(lc)){
+		if (lc->previewstream==NULL)
+			toggle_video_preview(lc,TRUE);
+	}else{
+		if (lc->previewstream!=NULL)
+			toggle_video_preview(lc,FALSE);
+	}
 #ifdef AMD
 	printf("iterate done...\n");
 #endif
@@ -1469,8 +1485,9 @@ int linphone_core_preview_ring(LinphoneCore *lc, const char *ring,LinphoneCoreCb
 	}
 	lc_callback_obj_init(&lc->preview_finished_cb,func,userdata);
 	lc->preview_finished=0;
-	if (lc->sound_conf.ring_sndcard!=NULL)
-		lc->ringstream=ring_start_with_cb((char*)ring,2000,lc->sound_conf.ring_sndcard,notify_end_of_ring,(void *)lc);
+	if (lc->sound_conf.ring_sndcard!=NULL){
+		lc->ringstream=ring_start_with_cb(ring,2000,lc->sound_conf.ring_sndcard,notify_end_of_ring,(void *)lc);
+	}
 	return 0;
 }
 
@@ -1619,8 +1636,7 @@ MSList * linphone_core_get_call_logs(LinphoneCore *lc){
 	return lc->call_logs;
 }
 
-void linphone_core_enable_video_preview(LinphoneCore *lc, bool_t val){
-	lc->video_conf.show_local=val;
+static void toggle_video_preview(LinphoneCore *lc, bool_t val){
 #ifdef VIDEO_ENABLED
 	if (lc->videostream==NULL){
 		if (val){
@@ -1643,7 +1659,7 @@ void linphone_core_enable_video(LinphoneCore *lc, bool_t val){
 		ms_warning("This version of linphone was built without video support.");
 #endif
 	lc->video_conf.enabled=val;
-	linphone_core_enable_video_preview(lc,val);
+	lc->video_conf.show_local=val;
 	/* need to re-apply network bandwidth settings*/
 	linphone_core_set_download_bandwidth(lc,
 		linphone_core_get_download_bandwidth(lc));
