@@ -419,7 +419,9 @@ typedef struct _LinphoneCore
 /* THE main API */
 
 void linphone_core_enable_logs(FILE *file);
-void linphone_core_disable_logs();
+void linphone_core_disable_logs(void);
+
+const char *linphone_core_get_version(void);
 
 LinphoneCore *linphone_core_new(const LinphoneCoreVTable *vtable,
 						const char *config_path, void* userdata);
