@@ -31,8 +31,12 @@
 #include "config.h"
 #endif
 
-#ifndef LINPHONE_VERSION 
-#define LINPHONE_VERSION "1.4.0"
+#ifndef LINPHONE_VERSION
+#define LINPHONE_VERSION "1.5.0"
+#endif
+
+#ifndef LIBLINPHONE_VERSION 
+#define LIBLINPHONE_VERSION LINPHONE_VERSION
 #endif
 
 #ifndef PACKAGE_SOUND_DIR 
