@@ -56,7 +56,7 @@ static pid_t pid = 0;
 
 /* Private functions */
 static void rcvregister_failure(int type, osip_transaction_t *tr,osip_message_t *sip);
-static int cb_udp_snd_message(osip_transaction_t *tr, osip_message_t *sip,
+int cb_udp_snd_message(osip_transaction_t *tr, osip_message_t *sip,
 			      char *host, int port, int out_socket);
 static void cb_ict_kill_transaction(int type, osip_transaction_t *tr);
 static void cb_ist_kill_transaction(int type, osip_transaction_t *tr);
@@ -1080,6 +1080,10 @@ static void cb_rcv2xx_4invite(osip_transaction_t *tr,osip_message_t *sip)
 	  }
       }
 
+	/* SM: do not send the ack now, just prepare it.
+	The application will send it using eXosip_send_ack(int jid)
+	*/
+	/*
     osip_message_get_route(ack, 0, &route);
     if (route!=NULL)
       {
@@ -1097,6 +1101,7 @@ static void cb_rcv2xx_4invite(osip_transaction_t *tr,osip_message_t *sip)
       }
 
     cb_udp_snd_message(NULL, ack, host, port, eXosip.j_socket);
+	*/
 
     jd->d_ack  = ack;
 
