@@ -139,7 +139,15 @@ the method index (_cnt_) and the argument size */
 #define MS_FILTER_ADD_ATTR		MS_FILTER_BASE_METHOD(8,const char)
 #define MS_FILTER_SET_MTU		MS_FILTER_BASE_METHOD(9,int)
 #define MS_FILTER_GET_MTU		MS_FILTER_BASE_METHOD(10,int)
-
+#define MS_FILTER_SET_FRAMESIZE MS_FILTER_BASE_METHOD(11,int)
+#define MS_FILTER_SET_FILTERLENGTH MS_FILTER_BASE_METHOD(12,int)
+#define MS_FILTER_SET_OUTPUT_SAMPLE_RATE MS_FILTER_BASE_METHOD(13,int)
+#define MS_FILTER_ENABLE_DIRECTMODE MS_FILTER_BASE_METHOD(14,int)
+#define MS_FILTER_ENABLE_VAD MS_FILTER_BASE_METHOD(15,int)
+#define MS_FILTER_GET_STAT_DISCARDED MS_FILTER_BASE_METHOD(16,int)
+#define MS_FILTER_GET_STAT_MISSED MS_FILTER_BASE_METHOD(17,int)
+#define MS_FILTER_GET_STAT_INPUT MS_FILTER_BASE_METHOD(18,int)
+#define MS_FILTER_GET_STAT_OUTPUT MS_FILTER_BASE_METHOD(19,int)
 
 /*private methods*/
 MSFilter *ms_filter_new_from_desc(MSFilterDesc *desc);
