@@ -55,11 +55,14 @@ void ms_queue_flush(MSQueue *q);
 
 void ms_queue_destroy(MSQueue *q);
 
-#define mblk_set_timestamp_info(m,ts)	(m)->reserved1=(ts);
-#define mblk_get_timestamp_info(m)	((m)->reserved1)
-#define mblk_set_marker_info(m,bit)	(m)->reserved2=(bit)
-#define mblk_get_marker_info(m)		(m)->reserved2	
-
+#define mblk_set_timestamp_info(m,ts) (m)->reserved1=(ts);
+#define mblk_get_timestamp_info(m)    ((m)->reserved1)
+#define mblk_set_marker_info(m,bit)   (m)->reserved2=((m)->reserved2|bit)
+#define mblk_get_marker_info(m)	      ((m)->reserved2&0x1)
+#define mblk_set_rate(m,bits)         (m)->reserved2=((m)->reserved2|(bits)<<1)
+#define mblk_get_rate(m)              (((m)->reserved2>>1)&0x3)
+#define mblk_set_payload_type(m,bits) (m)->reserved2=((m)->reserved2|(bits<<3))
+#define mblk_get_payload_type(m)      (((m)->reserved2>>3)&0xFF)
 
 struct _MSBufferizer{
 	queue_t q;
