@@ -38,7 +38,7 @@ struct _MSTicker
 	int exec_id;
 	uint32_t ticks;
 	uint64_t time;	/* a time since the start of the ticker expressed in milisec*/
-	int64_t orig; /* a relative time to take in account difference between time base given by consecutive get_cur_time_ptr() functions.*/
+	uint64_t orig; /* a relative time to take in account difference between time base given by consecutive get_cur_time_ptr() functions.*/
 	MSTickerTimeFunc get_cur_time_ptr;
 	void *get_cur_time_data;
 	bool_t run;       /* flag to indicate whether the ticker must be run or not */
@@ -76,6 +76,8 @@ void ms_ticker_destroy(MSTicker *ticker);
 
 void ms_ticker_set_time_func(MSTicker *ticker, MSTickerTimeFunc func, void *user_data);
 
+void ms_ticker_print_graphs(MSTicker *ticker);
+
 /* private functions:*/
 
 
