@@ -187,9 +187,9 @@ static int alsa_read(snd_pcm_t *handle,unsigned char *buf,int nsamples)
 			if (err<0) ms_warning("alsa_read: snd_pcm_readi() failed:%s.",snd_strerror(err));
 		}else if (err!=-EWOULDBLOCK){
 			ms_warning("alsa_read: snd_pcm_readi() failed:%s.",snd_strerror(err));
-		}else if (err==0){
-			ms_warning("alsa_read: snd_pcm_readi() returned 0");
 		}
+	}else if (err==0){
+		ms_warning("alsa_read: snd_pcm_readi() returned 0");
 	}
 	return err;
 }
@@ -448,8 +448,6 @@ static void alsa_card_detect(MSSndCardManager *m){
 		MSSndCard *card=alsa_card_new(i);
 		if (card!=NULL)
 			ms_snd_card_manager_add_card(m,card);
-		else
-			return;
 	}
 }
 
@@ -547,7 +545,11 @@ static bool_t alsa_can_read(snd_pcm_t *dev, int frames){
 
 void alsa_read_process(MSFilter *obj){
 	AlsaReadData *ad=(AlsaReadData*)obj->data;
+#ifdef AMD_HACK
+	int samples=(160*ad->rate)/8000;
+#else
 	int samples=(128*ad->rate)/8000;
+#endif
 	int err;
 	mblk_t *om=NULL;
 	if (ad->handle==NULL && ad->pcmdev!=NULL){
@@ -566,6 +568,9 @@ void alsa_read_process(MSFilter *obj){
 		om->b_wptr+=size;
 		/*ms_message("alsa_read_process: Outputing %i bytes",size);*/
 		ms_queue_put(obj->outputs[0],om);
+#ifdef AMD_HACK
+		break;
+#endif
 	}
 }
 
