@@ -17,7 +17,7 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
-#ifndef WIN32
+#if !defined(WIN32) && !defined(_WIN32_WCE)
 #include <sys/socket.h>
 #include <netdb.h>
 #endif
@@ -82,6 +82,7 @@ ice_sendtest( Socket myFd, StunAddress4 *dest,
 
 int ice_sound_send_stun_request(RtpSession *session, struct CandidatePair *remote_candidates, int round)
 {
+	int roll=250;
 #if 0
     /* in "passive" mode (UA not behind a NATor behind a full cone NAT),
     wait a few delay before sending the first STUN request:
@@ -95,10 +96,10 @@ int ice_sound_send_stun_request(RtpSession *session, struct CandidatePair *remot
 	if (remote_candidates==NULL)
 		return 0;
 
-    /* each 100 ms = 5*20 */
-    /* each 1 000 ms = 50*20 */
-    /* each 5 000 ms = 250*20 */
-    if (round%250==50)
+	if (round>500)
+		roll=2*roll;
+
+    if (round%roll==50)
     {
         int pos;
 
@@ -173,7 +174,7 @@ _ice_get_localip_for (struct sockaddr_storage *saddr, size_t saddr_len, char *lo
   int err, tmp;
   int sock;
   struct sockaddr_storage addr;
-  size_t addr_len;
+  socklen_t addr_len;
 
   strcpy (loc, "127.0.0.1");    /* always fallback to local loopback */
 
@@ -267,12 +268,22 @@ int ice_process_stun_message(RtpSession *session, struct CandidatePair *remote_c
         for (pos=0;pos<10 && remote_candidates[pos].remote_candidate.ipaddr[0]!='\0';pos++)
         {
             struct CandidatePair *cand_pair = &remote_candidates[pos];
+#ifdef RESTRICTIVE_ICE
             if (cand_pair->connectivity_check == VALID
                 ||cand_pair->connectivity_check == RECV_VALID)
             {
                 already_worked_once=pos;
                 break;
             }
+#else
+            if (cand_pair->connectivity_check == VALID
+                ||cand_pair->connectivity_check == RECV_VALID
+		||cand_pair->connectivity_check == SEND_VALID)
+            {
+                already_worked_once=pos;
+                break;
+            }
+#endif
         }
     }
 
@@ -417,17 +428,19 @@ int ice_process_stun_message(RtpSession *session, struct CandidatePair *remote_c
 
                         if (strncmp(msg.username.value, username, strlen(username))==0)
                         {
-#if 0
-							ms_message("ice.c: Valid STUN request received (to=%s:%i from=%s:%i).",
+#ifdef RESTRICTIVE_ICE
+                                ms_message("ice.c: Valid STUN request received (to=%s:%i from=%s:%i).",
                                 cand_pair->remote_candidate.ipaddr, cand_pair->remote_candidate.port,
                                 src6host, recvport);
-							/* We can't be sure the remote end will receive our answer:
-							connection could be only one way...
-							*/
-							if (cand_pair->connectivity_check != VALID)
+				/* We can't be sure the remote end will receive our answer:
+				connection could be only one way...
+				*/
+				if (cand_pair->connectivity_check != VALID)
                             {
                                 switch_to_address = pos;
                             }
+#else
+				switch_to_address = pos;
 #endif
                             if (cand_pair->connectivity_check == RECV_VALID
                                 || cand_pair->connectivity_check == VALID)
