@@ -160,8 +160,10 @@ static void sender_process(MSFilter * f)
 	mblk_t *im;
 	uint32_t timestamp;
 
-	if (s == NULL)
+	if (s == NULL){
+		ms_queue_flush(f->inputs[0]);
 		return;
+	}
 
 	while ((im = ms_queue_get(f->inputs[0])) != NULL) {
 		mblk_t *header;
@@ -170,7 +172,7 @@ static void sender_process(MSFilter * f)
 		ms_filter_lock(f);
 		if (d->dtmf != 0) {
 			rtp_session_send_dtmf(s, d->dtmf, timestamp);
-			ms_debug("RFC2833 dtmf sended.");
+			ms_debug("RFC2833 dtmf sent.");
 			d->dtmf = 0;
 			d->skip_until = timestamp + (3 * 160);
 			d->skip = TRUE;
@@ -184,7 +186,10 @@ static void sender_process(MSFilter * f)
 		}else{
 		  if (d->mute_mic==FALSE)
 		    {
+			int pt = mblk_get_payload_type(im);
 			header = rtp_session_create_packet(s, 12, NULL, 0);
+			if (pt>0)
+				rtp_set_payload_type(header, pt);
 			rtp_set_markbit(header, mblk_get_marker_info(im));
 			header->b_cont = im;
 			rtp_session_sendm_with_ts(s, header, timestamp);
@@ -330,6 +335,8 @@ static void receiver_process(MSFilter * f)
 
 		mblk_set_timestamp_info(payload, rtp_get_timestamp(m));
 		mblk_set_marker_info(payload, rtp_get_markbit(m));
+		mblk_set_payload_type(payload, rtp_get_payload_type(m));
+		
 		freeb(m);
 		ms_queue_put(f->outputs[0], payload);
 	}
