@@ -46,7 +46,9 @@ typedef struct V4lState{
 	char *mmapdbuf;
 	int msize;/*mmapped size*/
 	MSVideoSize vsize;
+	MSVideoSize got_vsize;
 	int pix_fmt;
+	int int_pix_fmt; /*internal pixel format */
 	mblk_t *frames[VIDEO_MAX_FRAME];
 	mblk_t *mire;
 	queue_t rq;
@@ -56,6 +58,7 @@ typedef struct V4lState{
 	float fps;
 	float start_time;
 	int frame_count;
+	int queued;
 	bool_t run;
 	bool_t usemire;
 	bool_t v4lv2; /*we interface with a V4Lv2 driver */
@@ -75,7 +78,7 @@ static bool_t v4lv2_try_format(V4lState *s, int fmtid){
 	fmt.fmt.pix.width       = s->vsize.width; 
 	fmt.fmt.pix.height      = s->vsize.height;
 	fmt.fmt.pix.pixelformat = fmtid;
-	/*fmt.fmt.pix.field       = V4L2_FIELD_INTERLACED;*/
+	fmt.fmt.pix.field = V4L2_FIELD_ANY;
 
         if (ioctl (s->fd, VIDIOC_S_FMT, &fmt)<0){
 		return FALSE;
@@ -86,8 +89,6 @@ static bool_t v4lv2_try_format(V4lState *s, int fmtid){
 static int v4lv2_configure(V4lState *s)
 {
 	struct v4l2_capability cap;
-	struct v4l2_cropcap cropcap;
-	struct v4l2_crop crop;
 	
         if (ioctl (s->fd, VIDIOC_QUERYCAP, &cap)<0) {
 		ms_message("Not a v4lv2 driver.");
@@ -104,24 +105,26 @@ static int v4lv2_configure(V4lState *s)
 		return -1;
 	}
 
-	cropcap.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
-	
-	ioctl(s->fd, VIDIOC_CROPCAP, &cropcap);
-
-	crop.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
-	crop.c = cropcap.defrect; /* reset to default */
-	
-	ioctl(s->fd, VIDIOC_S_CROP, &crop);
-	
 	if (v4lv2_try_format(s,V4L2_PIX_FMT_YUV420)){
 		s->pix_fmt=MS_YUV420P;
+		s->int_pix_fmt=V4L2_PIX_FMT_YUV420;
 		ms_message("v4lv2: YUV420P choosen");
-	}else if (v4lv2_try_format(s,V4L2_PIX_FMT_RGB24)){
-		s->pix_fmt=MS_RGB24;
-		ms_message("v4lv2: RGB24 choosen");
+	}else if (v4lv2_try_format(s,V4L2_PIX_FMT_NV12)){
+		s->pix_fmt=MS_YUV420P;
+		s->int_pix_fmt=V4L2_PIX_FMT_NV12;
+		ms_message("v4lv2: V4L2_PIX_FMT_NV12 choosen");
 	}else if (v4lv2_try_format(s,V4L2_PIX_FMT_MJPEG)){
 		s->pix_fmt=MS_MJPEG;
+		s->int_pix_fmt=V4L2_PIX_FMT_MJPEG;
 		ms_message("v4lv2: MJPEG choosen");
+	}else if (v4lv2_try_format(s,V4L2_PIX_FMT_YUYV)){
+		s->pix_fmt=MS_YUYV;
+		s->int_pix_fmt=V4L2_PIX_FMT_YUYV;
+		ms_message("v4lv2: V4L2_PIX_FMT_YUYV choosen");
+	}else if (v4lv2_try_format(s,V4L2_PIX_FMT_RGB24)){
+		s->pix_fmt=MS_RGB24;
+		s->int_pix_fmt=V4L2_PIX_FMT_RGB24;
+		ms_message("v4lv2: RGB24 choosen");
 	}else{
 		ms_error("Could not find supported pixel format.");
 		return -1;
@@ -174,6 +177,7 @@ static int v4lv2_do_mmap(V4lState *s){
 		s->frames[i]=msg;
 	}
 	s->frame_max=req.count;
+	/*
 	for (i = 0; i < s->frame_max; ++i) {
 		struct v4l2_buffer buf;
 
@@ -185,6 +189,7 @@ static int v4lv2_do_mmap(V4lState *s){
 			ms_error("VIDIOC_QBUF failed: %s",strerror(errno));
 		}
 	}
+	*/
 	/*start capture immediately*/
 	type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
 	if (-1 ==ioctl (s->fd, VIDIOC_STREAMON, &type)){
@@ -196,39 +201,53 @@ static int v4lv2_do_mmap(V4lState *s){
 
 static mblk_t * v4lv2_grab_image(V4lState *s){
 	struct v4l2_buffer buf;
-	unsigned int i,k;
+	unsigned int k;
 	memset(&buf,0,sizeof(buf));
+	mblk_t *ret=NULL;
 
 	buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
 	buf.memory = V4L2_MEMORY_MMAP;
-
-	if (ioctl(s->fd, VIDIOC_DQBUF, &buf)<0) {
-		switch (errno) {
-		case EAGAIN:
-			return 0;
-		case EIO:
-			/* Could ignore EIO, see spec. */
-		default:
-			ms_warning("VIDIOC_DQBUF failed: %s",strerror(errno));
+	if (s->queued){
+		if (ioctl(s->fd, VIDIOC_DQBUF, &buf)<0) {
+			switch (errno) {
+			case EAGAIN:
+			case EIO:
+				/* Could ignore EIO, see spec. */
+				break;
+			default:
+				ms_warning("VIDIOC_DQBUF failed: %s",strerror(errno));
+			}
+		}else{
+			s->queued--;
+			/*decrement ref count of dequeued buffer */
+			ret=s->frames[buf.index];
+			ret->b_datap->db_ref--;
+			if (buf.index >= s->frame_max){
+				ms_error("buf.index>=s->max_frames !");
+				return NULL;
+			}
+			if (buf.bytesused<=30){
+				ms_warning("Ignoring empty buffer...");
+				return NULL;
+			}
 		}
 	}
 
-	if (buf.index >= s->frame_max){
-		ms_error("buf.index>=s->max_frames !");
-		return NULL;
-	}
-
-	i=buf.index;
 	/*queue buffers whose ref count has dropped to 1, because they are not
 	still used anywhere in the filter chain */
 	for(k=0;k<s->frame_max;++k){
-		if (k!=i && s->frames[k]->b_datap->db_ref==1){
+		if (s->frames[k]->b_datap->db_ref==1){
 			buf.index=k;
 			if (-1==ioctl (s->fd, VIDIOC_QBUF, &buf))
-				ms_warning("VIDIOC_QBUF failed: %s",strerror(errno));
+				ms_warning("VIDIOC_QBUF %i failed: %s",k,  strerror(errno));
+			else {
+				/*increment ref count of queued buffer*/
+				s->frames[k]->b_datap->db_ref++;
+				s->queued++;
+			}
 		}
 	}
-	return s->frames[i];
+	return ret;
 }
 
 #endif
@@ -250,6 +269,7 @@ static void v4l_init(MSFilter *f){
 	s->frame_count=-1;
 	s->fps=15;
 	s->usemire=(getenv("DEBUG")!=NULL);
+	s->queued=0;
 	f->data=s;
 }
 
@@ -264,18 +284,20 @@ But unfortunately when using alsa dmix/asym plugins, some threads are started by
 */
 
 static int v4l_fd=-1;
+static bool_t reuse_fd=FALSE;
 static char *v4l_devname=NULL;
 
 static int v4l_start(MSFilter *f, void *arg)
 {
 	V4lState *s=(V4lState*)f->data;
 	int err=0;
-	if (v4l_fd>=0 ){
-		if (strcmp(v4l_devname,s->dev)==0){
+	if (v4l_fd>=0 && reuse_fd){
+		if (strcmp(v4l_devname,s->dev)==0 ){
 			/*use this one!*/
 			ms_message("v4l_start: reusing previous file descriptor.");
 			s->fd=v4l_fd;
 		}else{
+			ms_message("closing cached fd");
 			close(v4l_fd);
 			v4l_fd=-1;
 			ms_free(v4l_devname);
@@ -295,9 +317,10 @@ static int v4l_start(MSFilter *f, void *arg)
 		return -1;
 	}else{
 #ifdef HAVE_LINUX_VIDEODEV2_H
-		if (v4lv2_configure(s)<0)/*might not be V4Lv2*/
-#endif
+		if (v4lv2_configure(s)<0) {/*might not be V4LV2 */
+#else
 		{
+#endif
 			err=v4l_configure(s);
 			if (err<0) 
 			{
@@ -306,12 +329,12 @@ static int v4l_start(MSFilter *f, void *arg)
 				s->fd=-1;
 				return -1;
 			}
-		}
-#ifdef HAVE_LINUX_VIDEODEV2_H
-		else{
+			reuse_fd=TRUE;
+		}else{
+			ms_message("Device is a video4linux V2 one.");
 			s->v4lv2=TRUE;
+			reuse_fd=FALSE;
 		}
-#endif
 	}
 	return 0;
 }
@@ -326,13 +349,12 @@ static void v4l_start_capture(V4lState *s){
 static int v4l_stop(MSFilter *f, void *arg){
 	V4lState *s=(V4lState*)f->data;
 	if (s->fd>=0){
-		/* do not close*/
-		/*
 		ms_message("v4l fd %i closed",s->fd);
-		if (close(s->fd)<0){
-			ms_warning("MSV4l: Could not close(): %s",strerror(errno));
+		if (!reuse_fd){
+			if (close(s->fd)<0){
+				ms_warning("MSV4l: Could not close(): %s",strerror(errno));
+			}
 		}
-		*/
 		s->fd=-1;
 		s->frame_count=-1;
 	}
@@ -412,12 +434,44 @@ static int v4l_do_mmap(V4lState *s){
 	return 0;
 }
 
+static bool_t try_size(V4lState *s, MSVideoSize vsize){
+	struct video_window win;
+	int err;
+	memset(&win,0,sizeof(win));
+	/*set picture size */
+	win.x=win.y=0;
+	win.width=vsize.width;
+	win.height=vsize.height;
+	win.flags=0;
+	win.clips=NULL;
+	win.clipcount=0;
+
+	ms_message("Trying to set capture size to %ix%i", vsize.width,vsize.height);
+	err=ioctl(s->fd,VIDIOCSWIN,&win);
+	if (err<0){
+		ms_warning("Could not set window size: %s",strerror(errno));
+		return FALSE;
+	}
+
+	err=ioctl(s->fd, VIDIOCGWIN, &win);
+	if (err<0){
+		ms_warning("Could not get window size: %s",strerror(errno));
+		return FALSE;
+	}
+	if (s->vsize.width!=win.width || s->vsize.height!=win.height){
+		ms_warning("Capture size is not what we expected: asked for %ix%i and get %ix%i",vsize.width,vsize.height, win.width, win.height);
+	}
+	s->got_vsize.width=win.width;
+	s->got_vsize.height=win.height;
+	ms_message("Capture size set to %ix%i", s->got_vsize.width,s->got_vsize.height);
+	return TRUE;
+}
+
 static int v4l_configure(V4lState *s)
 {
 	struct video_channel chan;
 	struct video_picture pict;
 	struct video_capability cap;
-	struct video_window win;
 	int err;
 	int i;
 	int found=0;
@@ -425,7 +479,6 @@ static int v4l_configure(V4lState *s)
 	memset(&chan,0,sizeof(chan));
 	memset(&pict,0,sizeof(pict));
 	memset(&cap,0,sizeof(cap));
-	memset(&win,0,sizeof(win));
 
 	err=ioctl(s->fd,VIDIOCGCAP,&cap);
 	if (err!=0)
@@ -433,7 +486,8 @@ static int v4l_configure(V4lState *s)
 		ms_warning("MSV4l: cannot get device capabilities: %s.",strerror(errno));
 		return -1;
 	}
-	ms_message("Found %s device.",cap.name);
+
+	ms_message("Found %s device. (maxsize=%ix%i)",cap.name, cap.maxwidth, cap.maxheight);
 	for (i=0;i<cap.channels;i++)
 	{
 		chan.channel=i;
@@ -481,34 +535,23 @@ static int v4l_configure(V4lState *s)
 			ms_fatal("Unsupported video pixel format.");
 		}
 	}
-	/*set picture size */
-	win.x=win.y=0;
-	win.width=s->vsize.width;
-	win.height=s->vsize.height;
-	win.flags=0;
-	win.clips=NULL;
-	win.clipcount=0;
-
-	err=ioctl(s->fd,VIDIOCSWIN,&win);
-	if (err<0){
-		ms_error("Could not set window size: %s",strerror(errno));
+	if (!try_size(s,s->vsize)) {
+		if (!try_size(s,MS_VIDEO_SIZE_NS1)){
+			if (!try_size(s,MS_VIDEO_SIZE_VGA)){
+				if (!try_size(s,MS_VIDEO_SIZE_CIF)) {
+					if (!try_size(s,MS_VIDEO_SIZE_QCIF)) {
+						if (!try_size(s,MS_VIDEO_SIZE_QVGA)) {
+							return -1;
+						}
+					}
+				}
+			}
+		}
 	}
 	return 0;
 }	
 
 
-#define BPP 3
-static inline
-void crop( unsigned char *src, int s_width, int s_height, unsigned char *dest, int d_width, int d_height)
-{
-	register int i;
-	register int stride = d_width*BPP;
-	register unsigned char *s = src, *d = dest;
-	s += ((s_height - d_height)/2 * s_width * BPP) + ((s_width - d_width)/2 * BPP);
-	for (i = 0; i < d_height; i++, d += stride, s += s_width * BPP)
-		memcpy( d, s, stride);
-}
-
 int ms_to_v4l_pix_fmt(MSPixFmt p){
 	switch(p){
 		case MS_YUV420P:
@@ -521,14 +564,66 @@ int ms_to_v4l_pix_fmt(MSPixFmt p){
 	}
 }
 
+static void plane_copy(uint8_t *dest, int dw, int dh, uint8_t *src, int sw, int sh, int bpp){
+	int diffw=dw-sw;
+	int diffh=dh-sh;
+	int dlsize=dw*bpp;
+	int slsize=sw*bpp;
+	int i;
+	int dstartx=(diffw>0) ? diffw/2 : 0;
+	int dstarty=(diffh>0) ? diffh/2 : 0;
+	int sstartx=(diffw<0) ? diffw/2 : 0;
+	int sstarty=(diffh<0) ? diffh/2 : 0;
+	uint8_t *tmp1=dest;
+	uint8_t *tmp2=src;	
+
+	/* copy orig into dest */
+	tmp2+=sstarty*slsize;
+	tmp1+=dstarty*dlsize;
+	for(i=dstarty;i<dh-dstarty;++i){
+		memcpy(tmp1+(dstartx*bpp),tmp2+(sstartx*bpp),(sw-(2*sstartx))*bpp);
+		tmp1+=dlsize;
+		tmp2+=slsize;
+	}
+}
+
+static void pic_copy(uint8_t *dest, int dw, int dh, uint8_t *src, int sw, int sh, MSPixFmt fmt){
+	if (fmt==MS_YUV420P){
+		plane_copy(dest,dw,dh,src,sw,sh,1);
+		dest+=dw*dh;
+		src+=sw*sh;
+		plane_copy(dest,dw/2,dh/2,src,sw/2,sh/2,1);
+		dest+=dw*dh/4;
+		src+=sw*sh/4;
+		plane_copy(dest,dw/2,dh/2,src,sw/2,sh/2,1);
+	}else if (fmt==MS_RGB24){
+		plane_copy(dest,dw,dh,src,sw,sh,3);
+	}
+}
+
+static mblk_t *crop_or_pad(V4lState *s, mblk_t *pic){
+	int size=s->vsize.width*s->vsize.height;
+	mblk_t *newpic;
+	if (s->pix_fmt==MS_YUV420P) size=size*3/2;
+	else if (s->pix_fmt==MS_RGB24) size=size*3;
+	else ms_fatal("crop_or_pad: unsupported pixel format.");
+	newpic=allocb(size,0);
+	memset(newpic->b_wptr,0,size);
+	pic_copy(newpic->b_wptr, s->vsize.width, s->vsize.height,
+		pic->b_rptr,s->got_vsize.width,s->got_vsize.height,s->pix_fmt);
+	newpic->b_wptr+=size;
+	return newpic;
+}
+
 static mblk_t * v4l_grab_image_mmap(V4lState *s){
 	struct video_mmap vmap;
 	int err;
 	int syncframe;
 	int jitter=s->frame_max-1;
 	int query_frame;
-	vmap.width=s->vsize.width;
-	vmap.height=s->vsize.height;
+	mblk_t *ret;
+	vmap.width=s->got_vsize.width;
+	vmap.height=s->got_vsize.height;
 	vmap.format=ms_to_v4l_pix_fmt(s->pix_fmt);
 	
 	query_frame=(s->frame_ind) % s->frame_max;
@@ -557,18 +652,12 @@ static mblk_t * v4l_grab_image_mmap(V4lState *s){
 	}else {
 		return NULL;
 	}
-	/* not particularly efficient - hope for a capture source that 
-	   provides subcapture or setting window */
-	/*
-	if (obj->width != MS_VIDEO_SOURCE(obj)->width || obj->height != MS_VIDEO_SOURCE(obj)->height){
-		guchar tmp[obj->bsize];
-		crop((guchar*) obj->img[syncframe].buffer, obj->width, obj->height, tmp,
-			MS_VIDEO_SOURCE(obj)->width, MS_VIDEO_SOURCE(obj)->height);
-		memcpy(obj->img[syncframe].buffer, tmp, MS_VIDEO_SOURCE(obj)->width *
-			MS_VIDEO_SOURCE(obj)->height * obj->pict.depth/8);
+	ret=s->frames[syncframe];
+	/* crop or pad picture if obtained size is not what we want */
+	if (s->vsize.width!=s->got_vsize.width){
+		ret=crop_or_pad(s,ret);
 	}
-	*/
-	return s->frames[syncframe];
+	return ret;
 }
 
 static mblk_t * v4l_make_mire(V4lState *s){
@@ -646,14 +735,28 @@ static void v4l_purge(V4lState *s){
 	}
 }
 
+#ifdef HAVE_LINUX_VIDEODEV2_H
+static void v4lv2_purge(V4lState *s){
+	struct v4l2_buffer buf;
+	memset(&buf,0,sizeof(buf));
 
+	buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
+	buf.memory = V4L2_MEMORY_MMAP;
+	
+	for(;s->queued>0;s->queued--){
+		if (ioctl(s->fd, VIDIOC_DQBUF, &buf)==-1){
+			ms_warning("v4lv2_purge: Could not DQ buffer: %s",strerror(errno));
+		}
+	}
+}
+#endif
 
 static void v4l_do_munmap(V4lState *s){
 	int i;
 #ifdef HAVE_LINUX_VIDEODEV2_H
 	enum v4l2_buf_type type;
 	if (s->v4lv2){
-
+		v4lv2_purge(s);
 		/*stop capture immediately*/
 		type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
 		if (-1 ==ioctl (s->fd, VIDIOC_STREAMOFF, &type)){
@@ -721,7 +824,6 @@ static void *v4l_thread(void *ptr){
 
 static void v4l_process(MSFilter * obj){
 	V4lState *s=(V4lState*)obj->data;
-	mblk_t *m;
 	uint32_t timestamp;
 	int cur_frame;
 	if (s->frame_count==-1){
@@ -734,10 +836,7 @@ static void v4l_process(MSFilter * obj){
 		ms_mutex_lock(&s->mutex);
 		/*keep the most recent frame if several frames have been captured */
 		if (s->fd!=-1){
-			while((m=getq(&s->rq))!=NULL){
-				if (om!=NULL) freemsg(om);
-				om=m;
-			}
+			om=getq(&s->rq);
 		}else{
 			if (s->usemire)
 				om=dupmsg(v4l_make_mire(s));
@@ -753,9 +852,9 @@ static void v4l_process(MSFilter * obj){
 			mblk_set_marker_info(om,TRUE);
 			ms_queue_put(obj->outputs[0],om);
 			/*ms_message("picture sent");*/
+			s->frame_count++;
 		}
-		s->frame_count++;
-	}
+	}else flushq(&s->rq,0);
 }
 
 static void v4l_preprocess(MSFilter *f){
@@ -786,13 +885,20 @@ static int v4l_set_vsize(MSFilter *f, void *arg){
 	return 0;
 }
 
+static int v4l_get_vsize(MSFilter *f, void *arg){
+	V4lState *s=(V4lState*)f->data;
+	*(MSVideoSize*)arg=s->vsize;
+	return 0;
+}
+
 static MSFilterMethod methods[]={
 	{	MS_FILTER_SET_FPS	,	v4l_set_fps	},
 	{	MS_FILTER_GET_PIX_FMT	,	v4l_get_pix_fmt	},
-	{	MS_FILTER_SET_VIDEO_SIZE, v4l_set_vsize	},
-	{	MS_V4L_START			,	v4l_start		},
-	{	MS_V4L_STOP			,	v4l_stop		},
-	{	0								,	NULL			}
+	{	MS_FILTER_SET_VIDEO_SIZE, 	v4l_set_vsize	},
+	{	MS_V4L_START			,	v4l_start	},
+	{	MS_V4L_STOP			,	v4l_stop	},
+	{	MS_FILTER_GET_VIDEO_SIZE,	v4l_get_vsize },
+	{	0	,	NULL			}
 };
 
 MSFilterDesc ms_v4l_desc={
