@@ -103,8 +103,8 @@ static int enc_set_br(MSFilter *f, void *arg){
 	}else if (s->maxbr>=64000){
 		s->vsize.width=MS_VIDEO_SIZE_QCIF_W;
 		s->vsize.height=MS_VIDEO_SIZE_QCIF_H;
-		s->fps=5;
-		s->qmin=4;
+		s->fps=6;
+		s->qmin=5;
 	}
 	return 0;
 }
@@ -124,8 +124,10 @@ static bool_t parse_video_fmtp(const char *fmtp, float *fps, MSVideoSize *vsize)
 		int divider;
 		*equal='\0';
 		if (strcasecmp(tmp,"CIF")==0){
-			vsize->width=MS_VIDEO_SIZE_CIF_W;
-			vsize->height=MS_VIDEO_SIZE_CIF_H;
+			if (vsize->width>=MS_VIDEO_SIZE_CIF_W){
+				vsize->width=MS_VIDEO_SIZE_CIF_W;
+				vsize->height=MS_VIDEO_SIZE_CIF_H;
+			}
 		}else if (strcasecmp(tmp,"QCIF")==0){
 			vsize->width=MS_VIDEO_SIZE_QCIF_W;
 			vsize->height=MS_VIDEO_SIZE_QCIF_H;
@@ -135,7 +137,8 @@ static bool_t parse_video_fmtp(const char *fmtp, float *fps, MSVideoSize *vsize)
 		}
 		divider=atoi(equal+1);
 		if (divider!=0){
-			*fps=29.97/divider;
+			float newfps=29.97/divider;
+			if (*fps>newfps) *fps=newfps;
 		}else{
 			ms_warning("Could not find video fps");
 			ret=FALSE;
@@ -183,6 +186,7 @@ static void enc_mpeg4_init(MSFilter *f){
 static void prepare(EncState *s){
 	AVCodecContext *c=&s->av_context;
 	avcodec_get_context_defaults(c);
+	ms_message("sizeof(AVCodecContext)=%i",sizeof(AVCodecContext));
 	/* put codec parameters */
 	c->bit_rate=20000;
 	c->width = s->vsize.width;  
@@ -244,7 +248,7 @@ static void enc_preprocess(MSFilter *f){
 	else if (s->codec==CODEC_ID_MPEG4)
 		prepare_mpeg4(s);
 	else{
-		ms_error("Unsupported codec id !");
+		ms_error("Unsupported codec id %i",s->codec);
 		return;
 	}
 	s->av_codec=avcodec_find_encoder(s->codec);
