@@ -225,12 +225,13 @@ rtp_session_init (RtpSession * session, int mode)
 	if ((mode == RTP_SESSION_SENDONLY) || (mode == RTP_SESSION_SENDRECV))
 	{
 		rtp_session_set_flag (session, RTP_SESSION_SEND_NOT_STARTED);
-		session->send_ssrc=uint32_t_random();
+		session->snd.ssrc=uint32_t_random();
 		/* set default source description */
 		rtp_session_set_source_description(session,"unknown@unknown",NULL,NULL,
 				NULL,NULL,"oRTP-" ORTP_VERSION,"This is free sofware (LGPL) !");
 	}
-	session->telephone_events_pt=-1;	/* not defined a priori */
+	session->snd.telephone_events_pt=-1;	/* not defined a priori */
+	session->rcv.telephone_events_pt=-1;	/* not defined a priori */
 	rtp_session_set_profile (session, &av_profile); /*the default profile to work with */
 	session->rtp.socket=-1;
 	session->rtcp.socket=-1;
@@ -249,8 +250,8 @@ rtp_session_init (RtpSession * session, int mode)
 	rtp_signal_table_init (&session->on_timestamp_jump,session,"timestamp_jump");
 	rtp_signal_table_init (&session->on_network_error,session,"network_error");
 	rtp_signal_table_init (&session->on_rtcp_bye,session,"rtcp_bye");
-	wait_point_init(&session->send_wp);
-	wait_point_init(&session->recv_wp);
+	wait_point_init(&session->snd.wp);
+	wait_point_init(&session->rcv.wp);
 	/*defaults send payload type to 0 (pcmu)*/
 	rtp_session_set_send_payload_type(session,0);
 	/*sets supposed recv payload type to undefined */
@@ -360,10 +361,52 @@ rtp_session_set_blocking_mode (RtpSession * session, int yesno)
 void
 rtp_session_set_profile (RtpSession * session, RtpProfile * profile)
 {
-	session->profile = profile;
+	session->snd.profile = profile;
+	session->rcv.profile = profile;
 	rtp_session_telephone_events_supported(session);
 }
 
+
+/**
+ *rtp_session_set_send_profile:
+ *@session: a rtp session
+ *@profile: a rtp profile
+ *
+ *	Set the RTP profile to be used for the sending by this session. By default, all session are created by
+ *	rtp_session_new() are initialized with the AV profile, as defined in RFC 3551. The application
+ *	can set any other profile instead using that function.
+ *
+ *
+**/
+
+void
+rtp_session_set_send_profile (RtpSession * session, RtpProfile * profile)
+{
+	session->snd.profile = profile;
+	rtp_session_send_telephone_events_supported(session);
+}
+
+
+
+/**
+ *rtp_session_set_recv_profile:
+ *@session: a rtp session
+ *@profile: a rtp profile
+ *
+ *	Set the RTP profile to be used for the receiveing by this session. By default, all session are created by
+ *	rtp_session_new() are initialized with the AV profile, as defined in RFC 3551. The application
+ *	can set any other profile instead using that function.
+ *
+ *
+**/
+
+void
+rtp_session_set_recv_profile (RtpSession * session, RtpProfile * profile)
+{
+	session->rcv.profile = profile;
+	rtp_session_recv_telephone_events_supported(session);
+}
+
 /**
  *rtp_session_get_profile:
  *@session: a rtp session
@@ -372,7 +415,30 @@ rtp_session_set_profile (RtpSession * session, RtpProfile * profile)
  *
 **/
 RtpProfile *rtp_session_get_profile(RtpSession *session){
-	return session->profile;
+	return session->snd.profile;
+}
+
+
+/**
+ *rtp_session_get_send_profile:
+ *@session: a rtp session
+ *
+ *	Returns current send profile.
+ *
+**/
+RtpProfile *rtp_session_get_send_profile(RtpSession *session){
+	return session->snd.profile;
+}
+
+/**
+ *rtp_session_get_recv_profile:
+ *@session: a rtp session
+ *
+ *	Returns current receive profile.
+ *
+**/
+RtpProfile *rtp_session_get_recv_profile(RtpSession *session){
+	return session->rcv.profile;
 }
 
 /**
@@ -495,13 +561,13 @@ uint16_t rtp_session_get_seq_number(RtpSession *session){
 void
 rtp_session_set_ssrc (RtpSession * session, uint32_t ssrc)
 {
-	session->send_ssrc = ssrc;
+	session->snd.ssrc = ssrc;
 }
 
 
 void rtp_session_update_payload_type(RtpSession *session, int paytype){
 	/* check if we support this payload type */
-	PayloadType *pt=rtp_profile_get_payload(session->profile,paytype);
+	PayloadType *pt=rtp_profile_get_payload(session->rcv.profile,paytype);
 	session->hw_recv_pt=paytype;
 	if (pt!=0){
 		ortp_message ("payload type changed to %i(%s) !",
@@ -528,7 +594,7 @@ void rtp_session_update_payload_type(RtpSession *session, int paytype){
 int
 rtp_session_set_send_payload_type (RtpSession * session, int paytype)
 {
-	session->send_pt=paytype;
+	session->snd.pt=paytype;
 	return 0;
 }
 
@@ -539,7 +605,7 @@ rtp_session_set_send_payload_type (RtpSession * session, int paytype)
  *	Returns: the payload type currently used in outgoing rtp packets
 **/
 int rtp_session_get_send_payload_type(const RtpSession *session){
-	return session->send_pt;
+	return session->snd.pt;
 }
 
 /**
@@ -558,9 +624,9 @@ int
 rtp_session_set_recv_payload_type (RtpSession * session, int paytype)
 {
 	PayloadType *pt;
-	session->recv_pt=paytype;
+	session->rcv.pt=paytype;
 	session->hw_recv_pt=paytype;
-	pt=rtp_profile_get_payload(session->profile,paytype);
+	pt=rtp_profile_get_payload(session->rcv.profile,paytype);
 	if (pt!=NULL){
 		payload_type_changed(session,pt);
 	}
@@ -574,7 +640,7 @@ rtp_session_set_recv_payload_type (RtpSession * session, int paytype)
  *	Returns: the payload type currently used in incoming rtp packets
 **/
 int rtp_session_get_recv_payload_type(const RtpSession *session){
-	return session->recv_pt;
+	return session->rcv.pt;
 }
 
 /**
@@ -601,8 +667,8 @@ static void rtp_header_init_from_session(rtp_header_t *rtp, RtpSession *session)
 	rtp->extbit = 0;
 	rtp->markbit= 0;
 	rtp->cc = 0;
-	rtp->paytype = session->send_pt;
-	rtp->ssrc = session->send_ssrc;
+	rtp->paytype = session->snd.pt;
+	rtp->ssrc = session->snd.ssrc;
 	rtp->timestamp = 0;	/* set later, when packet is sended */
 	/* set a seq number */
 	rtp->seq_number=session->rtp.snd_seq;
@@ -754,14 +820,14 @@ rtp_session_sendm_with_ts (RtpSession * session, mblk_t *mp, uint32_t timestamp)
 				     session->rtp.snd_ts_offset) +
 					session->rtp.snd_time_offset;
 		/*ortp_message("rtp_session_send_with_ts: packet_time=%i time=%i",packet_time,sched->time_);*/
-		wait_point_lock(&session->send_wp);
+		wait_point_lock(&session->snd.wp);
 		if (TIME_IS_STRICTLY_NEWER_THAN (packet_time, sched->time_))
 		{
-			wait_point_wakeup_at(&session->send_wp,packet_time,(session->flags & RTP_SESSION_BLOCKING_MODE)!=0);	
+			wait_point_wakeup_at(&session->snd.wp,packet_time,(session->flags & RTP_SESSION_BLOCKING_MODE)!=0);	
 			session_set_clr(&sched->w_sessions,session);	/* the session has written */
 		}
 		else session_set_set(&sched->w_sessions,session);	/*to indicate select to return immediately */
-		wait_point_unlock(&session->send_wp);
+		wait_point_unlock(&session->snd.wp);
 	}
 	
 
@@ -770,7 +836,7 @@ rtp_session_sendm_with_ts (RtpSession * session, mblk_t *mp, uint32_t timestamp)
 	packsize = msgdsize(mp) ;
 	
 	rtp->timestamp=timestamp;
-	if (session->telephone_events_pt==rtp->paytype)
+	if (session->snd.telephone_events_pt==rtp->paytype)
 	{
 		session->rtp.snd_seq++;
 		rtp->seq_number = session->rtp.snd_seq;
@@ -830,7 +896,7 @@ extern void rtcp_parse(RtpSession *session, mblk_t *mp);
 
 
 static void payload_type_changed_notify(RtpSession *session, int paytype){
-	session->recv_pt = paytype;
+	session->rcv.pt = paytype;
 	rtp_signal_table_emit (&session->on_payload_type_changed);	
 }
 
@@ -914,7 +980,7 @@ rtp_session_recvm_with_ts (RtpSession * session, uint32_t user_ts)
 		/* remember the difference between the last received on the socket timestamp and the user timestamp */
 		session->rtp.rcv_diff_ts=session->rtp.hwrcv_diff_ts - session->rtp.jittctl.jitt_comp_ts;
 		session->rtp.rcv_last_ret_ts = user_ts;	/* just to have an init value */
-		session->recv_ssrc = rtp->ssrc;
+		session->rcv.ssrc = rtp->ssrc;
 		/* delete the recv synchronisation flag */
 		rtp_session_unset_flag (session, RTP_SESSION_RECV_SYNC);
 	}
@@ -941,7 +1007,7 @@ rtp_session_recvm_with_ts (RtpSession * session, uint32_t user_ts)
 		packet_ts=rtp->timestamp;
 		ortp_debug("Returning mp with ts=%i", packet_ts);
 		/* check for payload type changes */
-		if (session->recv_pt != rtp->paytype)
+		if (session->rcv.pt != rtp->paytype)
 		{
 			payload_type_changed_notify(session, rtp->paytype);
 		}
@@ -982,14 +1048,14 @@ rtp_session_recvm_with_ts (RtpSession * session, uint32_t user_ts)
 				     session->rtp.rcv_query_ts_offset) +
 			session->rtp.rcv_time_offset;
 		ortp_debug ("rtp_session_recvm_with_ts: packet_time=%i, time=%i",packet_time, sched->time_);
-		wait_point_lock(&session->recv_wp);
+		wait_point_lock(&session->rcv.wp);
 		if (TIME_IS_STRICTLY_NEWER_THAN (packet_time, sched->time_))
 		{
-			wait_point_wakeup_at(&session->recv_wp,packet_time, (session->flags & RTP_SESSION_BLOCKING_MODE)!=0);
+			wait_point_wakeup_at(&session->rcv.wp,packet_time, (session->flags & RTP_SESSION_BLOCKING_MODE)!=0);
 			session_set_clr(&sched->r_sessions,session);
 		}
 		else session_set_set(&sched->r_sessions,session);	/*to unblock _select() immediately */
-		wait_point_unlock(&session->recv_wp);
+		wait_point_unlock(&session->rcv.wp);
 	}
 	return mp;
 }
@@ -1076,10 +1142,10 @@ int rtp_session_recv_with_ts (RtpSession * session, uint8_t * buffer,
 	*have_more = 0;
 
 	mp = rtp_session_recvm_with_ts (session, ts);
-	payload =rtp_profile_get_payload (session->profile,
-					 session->recv_pt);
+	payload =rtp_profile_get_payload (session->rcv.profile,
+					 session->rcv.pt);
 	if (payload==NULL){
-		ortp_warning("rtp_session_recv_with_ts: unable to recv an unsupported payload.");
+		ortp_warning("rtp_session_recv_with_ts: unable to recv an unsupported payload (%i)",session->rcv.pt);
 		if (mp!=NULL) freemsg(mp);
 		return -1;
 	}
@@ -1174,8 +1240,8 @@ int rtp_session_recv_with_ts (RtpSession * session, uint8_t * buffer,
 			return 0;
 		}
 		mp = rtp_session_recvm_with_ts (session, ts);
-		payload = rtp_profile_get_payload (session->profile,
-						 session->recv_pt);
+		payload = rtp_profile_get_payload (session->rcv.profile,
+						 session->rcv.pt);
 		if (payload==NULL){
 			ortp_warning("rtp_session_recv_with_ts: unable to recv an unsupported payload.");
 			if (mp!=NULL) freemsg(mp);
@@ -1204,7 +1270,7 @@ uint32_t rtp_session_get_current_send_ts(RtpSession *session)
 	uint32_t session_time;
 	RtpScheduler *sched=session->sched;
 	PayloadType *payload;
-	payload=rtp_profile_get_payload(session->profile,session->send_pt);
+	payload=rtp_profile_get_payload(session->snd.profile,session->snd.pt);
 	return_val_if_fail(payload!=NULL, 0);
 	if ( (session->flags & RTP_SESSION_SCHEDULED)==0 ){
 		ortp_warning("can't guess current timestamp because session is not scheduled.");
@@ -1231,7 +1297,7 @@ uint32_t rtp_session_get_current_recv_ts(RtpSession *session){
 	uint32_t session_time;
 	RtpScheduler *sched=ortp_get_scheduler();
 	PayloadType *payload;
-	payload=rtp_profile_get_payload(session->profile,session->recv_pt);
+	payload=rtp_profile_get_payload(session->rcv.profile,session->rcv.pt);
 	return_val_if_fail(payload!=NULL, 0);
 	if ( (session->flags & RTP_SESSION_SCHEDULED)==0 ){
 		ortp_warning("can't guess current timestamp because session is not scheduled.");
@@ -1267,6 +1333,9 @@ void rtp_session_release_sockets(RtpSession *session){
 	if (session->rtcp.socket>=0) close_socket (session->rtcp.socket);
 	session->rtp.socket=-1;
 	session->rtcp.socket=-1;
+	session->rtp.tr = 0;
+	session->rtcp.tr = 0;
+
 	/* don't discard remote addresses, then can be preserved for next use.
 	session->rtp.rem_addrlen=0;
 	session->rtcp.rem_addrlen=0;
@@ -1274,11 +1343,11 @@ void rtp_session_release_sockets(RtpSession *session){
 }
 
 ortp_socket_t rtp_session_get_rtp_socket(const RtpSession *session){
-	return session->rtp.socket;
+	return rtp_session_using_transport(session, rtp) ? (session->rtp.tr->t_getsocket)(session->rtp.tr) : session->rtp.socket;
 }
 
 ortp_socket_t rtp_session_get_rtcp_socket(const RtpSession *session){
-	return session->rtcp.socket;
+	return rtp_session_using_transport(session, rtcp) ? (session->rtcp.tr->t_getsocket)(session->rtcp.tr) : session->rtcp.socket;
 }
 
 void rtp_session_register_event_queue(RtpSession *session, OrtpEvQueue *q){
@@ -1329,8 +1398,8 @@ void rtp_session_uninit (RtpSession * session)
 	/* close sockets */
 	rtp_session_release_sockets(session);
 
-	wait_point_uninit(&session->send_wp);
-	wait_point_uninit(&session->recv_wp);
+	wait_point_uninit(&session->snd.wp);
+	wait_point_uninit(&session->rcv.wp);
 	if (session->current_tev!=NULL) freemsg(session->current_tev);
 	if (session->rtp.cached_mp!=NULL) freemsg(session->rtp.cached_mp);
 	if (session->rtcp.cached_mp!=NULL) freemsg(session->rtcp.cached_mp);
@@ -1525,12 +1594,12 @@ rtp_session_get_last_recv_time(RtpSession *session, struct timeval *tv)
 uint32_t rtp_session_time_to_ts(RtpSession *session, int millisecs){
 	PayloadType *payload;
 	payload =
-		rtp_profile_get_payload (session->profile,
-					 session->send_pt);
+		rtp_profile_get_payload (session->snd.profile,
+					 session->snd.pt);
 	if (payload == NULL)
 	{
 		ortp_warning
-			("rtp_session_ts_to_t: use of unsupported payload type %d.", session->send_pt);
+			("rtp_session_ts_to_t: use of unsupported payload type %d.", session->snd.pt);
 		return 0;
 	}
 	/* the return value is in milisecond */
@@ -1542,12 +1611,12 @@ uint32_t rtp_session_ts_to_time (RtpSession * session, uint32_t timestamp)
 {
 	PayloadType *payload;
 	payload =
-		rtp_profile_get_payload (session->profile,
-					 session->send_pt);
+		rtp_profile_get_payload (session->snd.profile,
+					 session->snd.pt);
 	if (payload == NULL)
 	{
 		ortp_warning
-			("rtp_session_ts_to_t: use of unsupported payload type %d.", session->send_pt);
+			("rtp_session_ts_to_t: use of unsupported payload type %d.", session->snd.pt);
 		return 0;
 	}
 	/* the return value is in milisecond */
@@ -1560,18 +1629,18 @@ uint32_t rtp_session_ts_to_time (RtpSession * session, uint32_t timestamp)
 /* time is the number of miliseconds elapsed since the start of the scheduler */
 void rtp_session_process (RtpSession * session, uint32_t time, RtpScheduler *sched)
 {
-	wait_point_lock(&session->send_wp);
-	if (wait_point_check(&session->send_wp,time)){
+	wait_point_lock(&session->snd.wp);
+	if (wait_point_check(&session->snd.wp,time)){
 		session_set_set(&sched->w_sessions,session);
-		wait_point_wakeup(&session->send_wp);
+		wait_point_wakeup(&session->snd.wp);
 	}
-	wait_point_unlock(&session->send_wp);
+	wait_point_unlock(&session->snd.wp);
 	
-	wait_point_lock(&session->recv_wp);
-	if (wait_point_check(&session->recv_wp,time)){
+	wait_point_lock(&session->rcv.wp);
+	if (wait_point_check(&session->rcv.wp,time)){
 		session_set_set(&sched->r_sessions,session);
-		wait_point_wakeup(&session->recv_wp);
+		wait_point_wakeup(&session->rcv.wp);
 	}
-	wait_point_unlock(&session->recv_wp);
+	wait_point_unlock(&session->rcv.wp);
 }
 
