@@ -34,6 +34,16 @@
 #define USE_SENDMSG 1
 #endif
 
+#define can_connect(s)	( (s)->use_connect && !(s)->symmetric_rtp)
+
+static bool_t try_connect(int fd, const struct sockaddr *dest, socklen_t addrlen){
+	if (connect(fd,dest,addrlen)<0){
+		ortp_warning("Could not connect() socket: %s",getSocketError());
+		return FALSE;
+	}
+	return TRUE;
+}
+
 static ortp_socket_t create_and_bind(const char *addr, int port, int *sock_family){
 	int err;
 	int optval = 1;
@@ -53,7 +63,7 @@ static ortp_socket_t create_and_bind(const char *addr, int port, int *sock_famil
 	snprintf(num, sizeof(num), "%d",port);
 	err = getaddrinfo(addr,num, &hints, &res0);
 	if (err!=0) {
-		ortp_warning ("Error: %s", gai_strerror(err));
+		ortp_warning ("Error in getaddrinfo on (addr=%s port=%i): %s", addr, port, gai_strerror(err));
 		return -1;
 	}
 	
@@ -73,7 +83,7 @@ static ortp_socket_t create_and_bind(const char *addr, int port, int *sock_famil
 		err = bind (sock, res->ai_addr, res->ai_addrlen);
 		if (err != 0)
 		  {
-		    ortp_warning ("Fail to bind rtp socket to %s:%i : %s.", addr,port, getSocketError());
+		    ortp_warning ("Fail to bind rtp socket to (addr=%s port=%i) : %s.", addr,port, getSocketError());
 		    close_socket (sock);
 			sock=-1;
 		    continue;
@@ -582,8 +592,29 @@ rtp_session_set_remote_addr (RtpSession * session, const char * addr, int port)
 	memcpy (&session->rtcp.rem_addr, &session->rtp.rem_addr,
 		sizeof (struct sockaddr_in));
 	session->rtcp.rem_addr.sin_port = htons (port + 1);
-	session->rtcp.rem_addrlen=sizeof(session->rtp.rem_addr);
+	session->rtcp.rem_addrlen=sizeof(session->rtcp.rem_addr);
 #endif
+	if (can_connect(session)){
+		if (try_connect(session->rtp.socket,(struct sockaddr*)&session->rtp.rem_addr,session->rtp.rem_addrlen))
+			session->flags|=RTP_SOCKET_CONNECTED;
+		if (session->rtcp.socket>=0){
+			if (try_connect(session->rtcp.socket,(struct sockaddr*)&session->rtcp.rem_addr,session->rtcp.rem_addrlen))
+				session->flags|=RTCP_SOCKET_CONNECTED;
+		}
+	}else if (session->flags & RTP_SOCKET_CONNECTED){
+		/*must dissolve association done by connect().
+		See connect(2) manpage*/
+		struct sockaddr sa;
+		sa.sa_family=AF_UNSPEC;
+		if (connect(session->rtp.socket,&sa,sizeof(sa))<0){
+			ortp_error("Cannot dissolve connect() association for rtp socket: %s", getSocketError());
+		}
+		if (connect(session->rtcp.socket,&sa,sizeof(sa))<0){
+			ortp_error("Cannot dissolve connect() association for rtcp socket: %s", getSocketError());
+		}
+		session->flags&=~RTP_SOCKET_CONNECTED;
+		session->flags&=~RTCP_SOCKET_CONNECTED;
+	}
 	return 0;
 }
 
@@ -593,9 +624,22 @@ void rtp_session_set_sockets(RtpSession *session, int rtpfd, int rtcpfd)
 	if (rtcpfd>=0) set_non_blocking_socket(rtcpfd);
 	session->rtp.socket=rtpfd;
 	session->rtcp.socket=rtcpfd;
-	session->flags|=RTP_SESSION_USING_EXT_SOCKETS;
+	if (rtpfd>=0 || rtcpfd>=0 )
+		session->flags|=(RTP_SESSION_USING_EXT_SOCKETS|RTP_SOCKET_CONNECTED|RTCP_SOCKET_CONNECTED);
+	else session->flags&=~(RTP_SESSION_USING_EXT_SOCKETS|RTP_SOCKET_CONNECTED|RTCP_SOCKET_CONNECTED);
 }
 
+void rtp_session_set_transports(RtpSession *session, struct _RtpTransport *rtptr, struct _RtpTransport *rtcptr)
+{
+	session->rtp.tr = rtptr;
+	session->rtcp.tr = rtcptr;
+	if (rtptr || rtcptr )
+		session->flags|=(RTP_SESSION_USING_TRANSPORT);
+	else session->flags&=~(RTP_SESSION_USING_TRANSPORT);
+}
+
+
+
 /**
  *rtp_session_flush_sockets:
  *@session: a rtp session
@@ -614,6 +658,16 @@ void rtp_session_flush_sockets(RtpSession *session){
 	struct sockaddr from;
 #endif
 	socklen_t fromlen=sizeof(from);
+	if (rtp_session_using_transport(session, rtp))
+	  {
+	    while (session->rtp.tr->t_recvfrom(session->rtp.tr,trash,sizeof(trash),0,(struct sockaddr *)&from,&fromlen)>0){};
+
+	    if (session->rtcp.tr)
+	      while (session->rtcp.tr->t_recvfrom(session->rtcp.tr,trash,sizeof(trash),0,(struct sockaddr *)&from,&fromlen)>0){};
+	    
+	    return;
+	  }
+
 	if (session->rtp.socket>=0){
 		while (recvfrom(session->rtp.socket,trash,sizeof(trash),0,(struct sockaddr *)&from,&fromlen)>0){};
 	}
@@ -641,7 +695,6 @@ static int rtp_sendmsg(int sock,mblk_t *m, struct sockaddr *rem_addr, int addr_l
 	msg.msg_control=NULL;
 	msg.msg_controllen=0;
 	msg.msg_flags=0;
-	
 	error=sendmsg(sock,&msg,0);
 	return error;
 }
@@ -669,6 +722,9 @@ rtp_session_rtp_send (RtpSession * session, mblk_t * m)
 	int error;
 	int i;
 	rtp_header_t *hdr;
+	struct sockaddr *destaddr=(struct sockaddr*)&session->rtp.rem_addr;
+	socklen_t destlen=session->rtp.rem_addrlen;
+	ortp_socket_t sockfd=session->rtp.socket;
 
 	hdr = (rtp_header_t *) m->b_rptr;
 	/* perform host to network conversions */
@@ -677,29 +733,29 @@ rtp_session_rtp_send (RtpSession * session, mblk_t * m)
 	hdr->seq_number = htons (hdr->seq_number);
 	for (i = 0; i < hdr->cc; i++)
 		hdr->csrc[i] = htonl (hdr->csrc[i]);
-	
-#ifdef USE_SENDMSG
-	if (session->flags & RTP_SESSION_USING_EXT_SOCKETS){
-		error=rtp_sendmsg(session->rtp.socket,m,(struct sockaddr *)NULL,0);
-	}else {
-		error=rtp_sendmsg(session->rtp.socket,m,(struct sockaddr *) &session->rtp.rem_addr,
-		session->rtp.rem_addrlen);
+
+	if (session->flags & RTP_SOCKET_CONNECTED) {
+		destaddr=NULL;
+		destlen=0;
 	}
+#ifdef USE_SENDMSG
+	error=rtp_sendmsg(sockfd,m,destaddr,destlen);
 #else
 	if (m->b_cont!=NULL){
 		msgpullup(m,-1);
 	}
-	if (session->flags & RTP_SESSION_USING_EXT_SOCKETS){
-		error=send(session->rtp.socket, m->b_rptr, (int) (m->b_wptr - m->b_rptr),0);
-	}else error = sendto (session->rtp.socket, m->b_rptr,
-		(int) (m->b_wptr - m->b_rptr), 0,
-		(struct sockaddr *) &session->rtp.rem_addr,
-		session->rtp.rem_addrlen);
+
+	if (rtp_session_using_transport(session, rtp))
+	  error = (session->rtp.tr->t_sendto) (session->rtp.tr, m->b_rptr, (int) (m->b_wptr - m->b_rptr),
+			 0,destaddr,destlen);
+	else
+	  error = sendto (sockfd, m->b_rptr, (int) (m->b_wptr - m->b_rptr),
+			 0,destaddr,destlen);
 #endif
 	if (error < 0){
 		if (session->on_network_error.count>0){
 			rtp_signal_table_emit3(&session->on_network_error,(long)"Error sending RTP packet",INT_TO_POINTER(getSocketErrorCode()));
-		}else ortp_warning ("Error sending rtp packet: %s ; socket=%i", getSocketError(), session->rtp.socket);
+		}else ortp_warning ("Error sending rtp packet: %s ; socket=%i", getSocketError(), sockfd);
 	}else{
 		update_sent_bytes(session,error);
 	}
@@ -711,48 +767,49 @@ int
 rtp_session_rtcp_send (RtpSession * session, mblk_t * m)
 {
 	int error=0;
-	bool_t using_ext_socket=(session->flags & RTP_SESSION_USING_EXT_SOCKETS)!=0;
-	if ( (using_ext_socket && session->rtcp.socket>0 ) || session->rtcp.rem_addrlen>0){
+	ortp_socket_t sockfd=session->rtcp.socket;
+	struct sockaddr *destaddr=(struct sockaddr*)&session->rtcp.rem_addr;
+	socklen_t destlen=session->rtcp.rem_addrlen;
+	bool_t using_connected_socket=(session->flags & RTCP_SOCKET_CONNECTED)!=0;
+
+	if (using_connected_socket) {
+		destaddr=NULL;
+		destlen=0;
+	}
+
+	if ((sockfd>=0 || rtp_session_using_transport(session, rtcp)) && (session->rtcp.rem_addrlen>0 ||using_connected_socket) ){
 		
 #ifndef USE_SENDMSG
 		if (m->b_cont!=NULL){
 			msgpullup(m,-1);
 		}
-#endif
-		if (using_ext_socket &&  session->rtcp.socket>0 ){
-#ifdef USE_SENDMSG
-			error=rtp_sendmsg(session->rtcp.socket,m,(struct sockaddr *)NULL,0);
-#else
-			error=send(session->rtcp.socket, m->b_rptr, (int) (m->b_wptr - m->b_rptr),0);
-#endif
-		}else {
-#ifdef USE_SENDMSG
-			error=rtp_sendmsg(session->rtcp.socket,m,(struct sockaddr *) &session->rtcp.rem_addr,
-			session->rtcp.rem_addrlen);
-#else
-			error = sendto (session->rtcp.socket, m->b_rptr,
+		if (rtp_session_using_transport(session, rtcp))
+		  error = (session->rtcp.tr->t_sendto) (sockfd, m->b_rptr,
 			(int) (m->b_wptr - m->b_rptr), 0,
-			(struct sockaddr *) &session->rtcp.rem_addr,
-			session->rtcp.rem_addrlen);
+			destaddr, destlen);
+		else
+		  error = sendto (sockfd, m->b_rptr,
+			(int) (m->b_wptr - m->b_rptr), 0,
+			destaddr, destlen);
+#else
+		error=rtp_sendmsg(sockfd,m,destaddr, destlen);
 #endif
-		}
-	
 		if (error < 0){
 			char host[50];
 			if (session->on_network_error.count>0){
 				rtp_signal_table_emit3(&session->on_network_error,(long)"Error sending RTCP packet",INT_TO_POINTER(getSocketErrorCode()));
 			}else ortp_warning ("Error sending rtcp packet: %s ; socket=%i; addr=%s", getSocketError(), session->rtcp.socket, ortp_inet_ntoa((struct sockaddr*)&session->rtcp.rem_addr,session->rtcp.rem_addrlen,host,sizeof(host)) );
 		}
-	}else ortp_warning("Cannot send rtcp report because I don't know the remote address.");
+	}else ortp_warning("Cannot send rtcp report: sockfd=%i, rem_addrlen=%i, connected=%i",sockfd,session->rtcp.rem_addrlen,using_connected_socket);
 	freemsg (m);
 	return error;
 }
 
-
 int
 rtp_session_rtp_recv (RtpSession * session, uint32_t user_ts)
 {
 	int error;
+	ortp_socket_t sockfd=session->rtp.socket;
 #ifdef ORTP_INET6
 	struct sockaddr_storage remaddr;
 #else
@@ -760,10 +817,9 @@ rtp_session_rtp_recv (RtpSession * session, uint32_t user_ts)
 #endif
 	socklen_t addrlen = sizeof (remaddr);
 	mblk_t *mp;
-	RtpStream *stream=&session->rtp;
+	bool_t sock_connected=(session->flags & RTP_SOCKET_CONNECTED);
 
-	if (session->rtp.socket<0) return -1;  /*session has no sockets for the moment*/
-	
+	if ((sockfd<0) && !rtp_session_using_transport(session, rtp)) return -1;  /*session has no sockets for the moment*/
 
 	while (1)
 	{
@@ -772,32 +828,32 @@ rtp_session_rtp_recv (RtpSession * session, uint32_t user_ts)
 			 session->rtp.cached_mp = allocb (session->recv_buf_size, 0);
 		mp=session->rtp.cached_mp;
 		bufsz=(int) (mp->b_datap->db_lim - mp->b_datap->db_base);
-		if (session->flags & RTP_SESSION_USING_EXT_SOCKETS){
-			error=recv(session->rtp.socket,mp->b_wptr,bufsz,0);
-		}else error = recvfrom (session->rtp.socket, mp->b_wptr,
+		if (sock_connected){
+			error=recv(sockfd,mp->b_wptr,bufsz,0);
+		}else if (rtp_session_using_transport(session, rtp)) 
+		  error = (session->rtp.tr->t_recvfrom)(session->rtp.tr, mp->b_wptr,
 				  bufsz, 0,
 				  (struct sockaddr *) &remaddr,
 				  &addrlen);
-		if (error > 0)
-		{
-			if (error<RTP_FIXED_HEADER_SIZE){
-				ortp_warning("Packet too small to be a rtp packet (%i)!",error);
-				stream->stats.bad++;
-				ortp_global_stats.bad++;
-				/* don't free, it will be reused next time */
-			}else{
-				/* then parse the message and put on queue */
-				mp->b_wptr+=error;
-				rtp_session_rtp_parse (session, mp, user_ts + session->rtp.hwrcv_diff_ts, (struct sockaddr*)&remaddr,addrlen);
-				session->rtp.cached_mp=NULL;
-
-				if (session->symmetric_rtp==1 && addrlen>0){
-					/* store the sender rtp address to do symmetric RTP */
-					memcpy(&session->rtp.rem_addr,&remaddr,addrlen);
-					session->rtp.rem_addrlen=addrlen;
+		else error = recvfrom(sockfd, mp->b_wptr,
+				  bufsz, 0,
+				  (struct sockaddr *) &remaddr,
+				  &addrlen);
+		if (error > 0){
+			if (session->symmetric_rtp && !sock_connected){
+				/* store the sender rtp address to do symmetric RTP */
+				memcpy(&session->rtp.rem_addr,&remaddr,addrlen);
+				session->rtp.rem_addrlen=addrlen;
+				if (session->use_connect){
+					if (try_connect(sockfd,(struct sockaddr*)&remaddr,addrlen))
+						session->flags|=RTP_SOCKET_CONNECTED;
 				}
-
 			}
+			/* then parse the message and put on queue */
+			mp->b_wptr+=error;
+			rtp_session_rtp_parse (session, mp, user_ts + session->rtp.hwrcv_diff_ts, (struct sockaddr*)&remaddr,addrlen);
+			session->rtp.cached_mp=NULL;
+			/*for bandwidth measurements:*/
 			update_recv_bytes(session,error);
 		}
 		else
@@ -829,6 +885,8 @@ void rtp_session_notify_inc_rtcp(RtpSession *session, mblk_t *m){
 		d->packet=m;
 		rtp_session_dispatch_event(session,ev);
 	}
+	else
+	  freemsg(m);  /* avoid memory leak */
 }
 
 int
@@ -842,9 +900,9 @@ rtp_session_rtcp_recv (RtpSession * session)
 #endif
 	socklen_t addrlen=0;
 	mblk_t *mp;
-	
+	bool_t sock_connected=session->flags & RTCP_SOCKET_CONNECTED;
 
-	if (session->rtcp.socket<0) return -1;  /*session has no rtcp sockets for the moment*/
+	if (session->rtcp.socket<0 && !rtp_session_using_transport(session, rtcp)) return -1;  /*session has no rtcp sockets for the moment*/
 	
 
 	while (1)
@@ -853,11 +911,18 @@ rtp_session_rtcp_recv (RtpSession * session)
 			 session->rtcp.cached_mp = allocb (RTCP_MAX_RECV_BUFSIZE, 0);
 		
 		mp=session->rtcp.cached_mp;
-		if (session->flags & RTP_SESSION_USING_EXT_SOCKETS){
+		if (sock_connected){
 			error=recv(session->rtcp.socket,mp->b_wptr,RTCP_MAX_RECV_BUFSIZE,0);
 		}else {
 			addrlen=sizeof (remaddr);
-			error=recvfrom (session->rtcp.socket, mp->b_wptr,
+
+			if (rtp_session_using_transport(session, rtcp))
+			  error=(session->rtcp.tr->t_recvfrom)(session->rtcp.tr, mp->b_wptr,
+				  RTCP_MAX_RECV_BUFSIZE, 0,
+				  (struct sockaddr *) &remaddr,
+				  &addrlen);
+			else
+			  error=recvfrom (session->rtcp.socket, mp->b_wptr,
 				  RTCP_MAX_RECV_BUFSIZE, 0,
 				  (struct sockaddr *) &remaddr,
 				  &addrlen);
@@ -870,14 +935,14 @@ rtp_session_rtcp_recv (RtpSession * session)
 				rtp_session_notify_inc_rtcp(session,mp);
 			}
 			session->rtcp.cached_mp=NULL;
-			if (session->symmetric_rtp==1 && addrlen>0){
-				/* store the sender rtcp address to send him receiver reports */
-				memcpy(&session->rtcp.rem_addr,&remaddr,addrlen);
-				session->rtcp.rem_addrlen=addrlen;
-			} else if (session->rtcp.rem_addrlen==0 && addrlen>0){
-				/* store the sender rtcp address to send him receiver reports */
+			if (session->symmetric_rtp && !sock_connected){
+				/* store the sender rtp address to do symmetric RTP */
 				memcpy(&session->rtcp.rem_addr,&remaddr,addrlen);
 				session->rtcp.rem_addrlen=addrlen;
+				if (session->use_connect){
+					if (try_connect(session->rtcp.socket,(struct sockaddr*)&remaddr,addrlen))
+						session->flags|=RTCP_SOCKET_CONNECTED;
+				}
 			}
 		}
 		else
